from pydantic import SecretStr
from pydantic_settings import BaseSettings, SettingsConfigDict

from lemonaid.enums import CitraEnvironment


class CitraClientSettings(BaseSettings):
    model_config = SettingsConfigDict(
        env_prefix="CITRA_",
        case_sensitive=False,
    )
    token: SecretStr = SecretStr("")
    environment: CitraEnvironment = CitraEnvironment.PRODUCTION
    base_url: str = "api.citra.space"

    @property
    def url(self) -> str:
        env_prefix = ""
        if self.environment == CitraEnvironment.DEVELOPMENT:
            env_prefix = "dev."
        return f"https://{env_prefix}{self.base_url}"  # noqa: E231

    @property
    def authorization_header(self) -> dict[str, str]:
        return {"Authorization": f"Bearer {self.token.get_secret_value()}"}



import requests  # type: ignore

from lemonaid.schemas import ElsetReadList
from lemonaid.settings import CITRA_CLIENT_SETTINGS


class _CitraElsetClient:
    @staticmethod
    def latest() -> ElsetReadList:
        url = f"{CITRA_CLIENT_SETTINGS.url}/elsets/latest"
        headers = CITRA_CLIENT_SETTINGS.authorization_header
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        return ElsetReadList.model_validate(response.json())



from pydantic import SecretStr

from lemonaid.client._citra_elset_client import _CitraElsetClient
from lemonaid.enums import CitraEnvironment
from lemonaid.settings import CITRA_CLIENT_SETTINGS


class CitraClient:
    def __init__(self, token: str | None = None, environment: CitraEnvironment | None = None):
        if token:
            CITRA_CLIENT_SETTINGS.token = SecretStr(token)
        if environment:
            CITRA_CLIENT_SETTINGS.environment = environment

    elsets = _CitraElsetClient



from datetime import datetime
from uuid import UUID

from pydantic import RootModel

from lemonaid.enums import CitraElsetType
from lemonaid.schemas._citra_base_model import CitraBaseModel


class ElsetRead(CitraBaseModel):

    id: UUID
    creation_epoch: datetime
    epoch: datetime
    type: CitraElsetType
    semi_major_axis: float
    inclination: float
    raan: float
    eccentricity: float
    argument_of_perigee: float
    mean_anomaly: float
    mean_motion_dot: float = 0.0
    mean_motion_dot_dot: float = 0.0
    b_star: float = 0.0
    ballistic_coefficient: float = 0.0
    srp_coefficient: float = 0.0
    rms: float | None = None


class ElsetReadList(RootModel[list[ElsetRead]]):
    pass



from enum import Enum


class CitraElsetType(Enum):
    MEAN_KOZAI_GP = "SGP4 with Kozai mean motion"
    MEAN_BROUWER_GP = "SGP4 with Brouwer mean motion"
    MEAN_BROUWER_XP = "SGP4-XP with Brouwer mean motion"
    OSCULATING = "SP with Brouwer mean motion"
